#pragma once

#include "ofMain.h"
#include "ofxiPhone.h"
#include "ofxiPhoneExtras.h"

#import "PGMidi.h"

#include <deque>

class ofxMidiContext {

	public:
	
		static void setup() {
			if(isSetup())
				return;
			midi = [[PGMidi alloc] init];
		}
	
		static bool isSetup() {return midi != NULL;}
		static PGMidi * getMidi() {return midi;}

		static void listSources() {
			cout << [midi.sources count] << " Sources" << endl;
			for(PGMidiSource * source in midi.sources) {
				cout << "\t" << [source.name UTF8String] << endl;
			}
		}
		
		static void listDestinations() {
			cout << [midi.sources count] << " Destinations" << endl;
			for(PGMidiDestination * dest in midi.destinations) {
				cout << "\t" << [dest.name UTF8String] << endl;
			}
		}
		
		static void enableNetwork(bool network) {
			if(network)
				[midi enableNetwork:YES];
			else
				[midi enableNetwork:NO];
		}

	private:

		static PGMidi * midi;
};


@class PGMidi;
@class PGMidiSource;
@class PGMidiSourceDelegate;

class testApp;

@interface ofxMidiDelegate : NSObject <PGMidiDelegate, PGMidiSourceDelegate> {
    PGMidi * midi;
	testApp * app;
	unsigned long long lastTime;
	bool firstPacket;
}

- (void) midiSource:(PGMidiSource *)input midiReceived:(const MIDIPacketList *)packetList;

- (void) midi:(PGMidi*)midi sourceAdded:(PGMidiSource *)source;
- (void) midi:(PGMidi*)midi sourceRemoved:(PGMidiSource *)source;
- (void) midi:(PGMidi*)midi destinationAdded:(PGMidiDestination *)destination;
- (void) midi:(PGMidi*)midi destinationRemoved:(PGMidiDestination *)destination;

- (void) attachToAllExistingSources;
- (void) setMidi:(PGMidi *)m;
- (void) setApp:(void *)a;

- (void) listSources;
- (void) listDestinations;

@property (nonatomic,strong) PGMidi * midi;

@end

class testApp : public ofxiPhoneApp {
	
public:
	void setup();
	void update();
	void draw();
	void exit();
	
	void touchDown(ofTouchEventArgs &touch);
	void touchMoved(ofTouchEventArgs &touch);
	void touchUp(ofTouchEventArgs &touch);
	void touchDoubleTap(ofTouchEventArgs &touch);
	void touchCancelled(ofTouchEventArgs &touch);

	void lostFocus();
	void gotFocus();
	void gotMemoryWarning();
	void deviceOrientationChanged(int newOrientation);

	void addMessage(string msg);

	ofxMidiDelegate * midiDelegate;
	
	deque<string> messages;
	int maxMessages;
	
	int note, ctl;
	vector<unsigned char> bytes;
};


